
#ifndef __PROFILER_H__
#define __PROFILER_H__

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the PROFILER_EXPORTS
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// PROFILER_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef PROFILER_EXPORTS
#define PROFILER_API __declspec(dllexport)
#else
#define PROFILER_API __declspec(dllimport)
#endif

//! Function to provide _penter function hook as a result of /Gh compiler option.
//! This function will be called at the start of every method or function.
extern "C" void PROFILER_API _cdecl _penter( void );

//! Function to provide _pexit function hook as a result of /GH compiler option.
//! This function will be called at the end of every method or function.
extern "C" void PROFILER_API _cdecl _pexit( void );


#define __PROFILER_ENABLED__
#define __PROFILER_FULL_TYPE_EXPANSION__

#undef noinline
#undef fastcall

#if defined(_MSC_VER)
	#undef __PRETTY_FUNCTION__
	#define __PRETTY_FUNCTION__ __FUNCSIG__
	#define PROFILE_CONCAT( a, b ) a "/" b

	#define noinline __declspec(noinline)
	#define fastcall __fastcall
#else
	#define PROFILE_CONCAT( a, b ) b

	#define noinline __attribute__ ((noinline))
	#define fastcall
#endif

#if defined(__PROFILER_FULL_TYPE_EXPANSION__)
	#define PROFILE_FUNCTION() __PRETTY_FUNCTION__
#else
	#define PROFILE_FUNCTION() __FUNCTION__
#endif

#if defined(__PROFILER_ENABLED__)
	// start and stop the whole thing
	#define PROFILE_INIT ::Profiler::detect( GetCommandLine() )
	#define PROFILE_DUMP_REPORT_AS_HTML ::Profiler::dumphtml( )
	
	#define PROFILE_ADD_TO_HICCUP_STAT ::Profiler::addtohiccupstat( )
	#define PROFILE_ADD_TO_NOT_HICCUP_STAT ::Profiler::addtonothiccupstat( )

	#define PROFILE_START_PROFILING ::Profiler::startprofiling()
	#define PROFILE_STOP_PROFILING ::Profiler::stopprofiling()

	// thread
	#define PROFILE_THREAD_START_RAW( text )   ::Profiler::threadenter( text );
	#define PROFILE_THREAD_START()             PROFILE_THREAD_START_RAW( PROFILE_FUNCTION()  )
	#define PROFILE_THREAD_START_DESC( desc )  PROFILE_THREAD_START_RAW( PROFILE_CONCAT( PROFILE_FUNCTION(), desc ) )

	#define PROFILE_THREAD_SCOPED_RAW( text )  ::Profiler::ScopedThread profiler##__LINE__ ( text );
	#define PROFILE_THREAD_SCOPED()            PROFILE_THREAD_SCOPED_RAW( PROFILE_FUNCTION() )
	#define PROFILE_THREAD_SCOPED_DESC( desc ) PROFILE_THREAD_SCOPED_RAW( PROFILE_CONCAT( PROFILE_FUNCTION(), desc ) )

	#define PROFILE_THREAD_STOP()              ::Profiler::threadexit();

	// function
	#define PROFILE_PAUSE()             ::Profiler::pause();
	#define PROFILE_UNPAUSE()           ::Profiler::unpause();
	#define PROFILE_PAUSE_SCOPED()      ::Profiler::ScopedPause profilerpause##__LINE__;

	#define PROFILE_START_RAW( text )   ::Profiler::enter( text );
	#define PROFILE_START()             PROFILE_START_RAW( PROFILE_FUNCTION()  )
	#define PROFILE_START_DESC( desc )  PROFILE_START_RAW( PROFILE_CONCAT( PROFILE_FUNCTION(), desc ) )

	#define PROFILE_SCOPED_RAW( text )  ::Profiler::Scoped profiler##__LINE__ ( text );
	#define PROFILE_SCOPED()            PROFILE_SCOPED_RAW( PROFILE_FUNCTION() )
	#define PROFILE_SCOPED_DESC( desc ) PROFILE_SCOPED_RAW( PROFILE_CONCAT( PROFILE_FUNCTION(), desc ) )

	#define PROFILE_STOP()              ::Profiler::exit();
#else
	#define PROFILE_ADD_TO_HICCUP_STAT
	#define PROFILE_ADD_TO_NOT_HICCUP_STAT

	#define PROFILE_START_PROFILING 
	#define PROFILE_STOP_PROFILING 

	#define PROFILE_INIT
	#define PROFILE_DUMP_REPORT_AS_HTML

	#define PROFILE_THREAD_START_RAW( text )
	#define PROFILE_THREAD_START()
	#define PROFILE_THREAD_START_DESC( desc )

	#define PROFILE_THREAD_SCOPED_RAW( text )
	#define PROFILE_THREAD_SCOPED()
	#define PROFILE_THREAD_SCOPED_DESC( desc )

	#define PROFILE_THREAD_STOP()

	#define PROFILE_PAUSE()
	#define PROFILE_UNPAUSE()
	#define PROFILE_PAUSE_SCOPED()

	#define PROFILE_START_RAW( text )
	#define PROFILE_START()
	#define PROFILE_START_DESC( desc )

	#define PROFILE_SCOPED_RAW( text )
	#define PROFILE_SCOPED()
	#define PROFILE_SCOPED_DESC( desc )

	#define PROFILE_STOP()
#endif

#if defined(__PROFILER_ENABLED__)
namespace Profiler {
	/*
	=============
	Types that won't conflict with the rest of the system
	=============
	*/
	typedef float f32;
	typedef double f64;
	typedef unsigned char u8;
	typedef unsigned short u16;
	typedef unsigned int u32;
	#if defined(_MSC_VER)
		typedef unsigned __int64 u64;
	#else
		typedef unsigned long long u64;
	#endif

	template< class type1, class type2 >
	f64 average( type1 sum, type2 count ) {
		return ( count ) ? f64(sum)/f64(count) : 0;
	}



	/*
	=============
	Interface functions
	=============
	*/
	void PROFILER_API  init();
	void PROFILER_API  deInit();
	void PROFILER_API  detect( int argc, const char *argv[] );
	void PROFILER_API  detect( const char *commandLine );
	void PROFILER_API  dump();
	void PROFILER_API  dumphtml();
	void PROFILER_API  fastcall enter( const char *name );
	void PROFILER_API  fastcall exit();
	void PROFILER_API  fastcall pause();
	void PROFILER_API  PROFILER_API  fastcall unpause();
	void PROFILER_API  threadenter( const char *name );
	void PROFILER_API  threadexit();
	void PROFILER_API  reset();

	void PROFILER_API  addtohiccupstat( );
	void PROFILER_API  addtonothiccupstat( );
	void PROFILER_API  startprofiling( );
	void PROFILER_API  stopprofiling( );

	void PROFILER_API  enterFunc( unsigned long va);
	void PROFILER_API exitFunc( unsigned long va);

}; // namespace Profiler



#endif

#endif // __PROFILER_H__
